%define mpiimpl openmpi
%define mpidir %_libexecdir/%mpiimpl

%define oname dolfin
%define scalar_type real
%define ldir %_libexecdir/petsc-%scalar_type
Name: %oname-%scalar_type
Version: 0.9.2
Release: alt5.hg20090831
Summary: C++/Python library for solving differential equations
License: LGPL v2.1
Group: Sciences/Mathematics
Url: http://www.fenics.org/
Packager: Eugeny A. Rostovtsev (REAL) <real at altlinux.org>

# http://www.fenics.org/hg/dolfin
Source: %oname-%version.tar.gz

Requires: python-module-%name = %version-%release
Requires: python-module-%{name}_utils = %version-%release
Requires: lib%name = %version-%release
BuildPreReq: %mpiimpl-devel python-devel
BuildPreReq: scons swig libgts-devel libxml2-devel python-module-numpy
BuildPreReq: boost-devel libvtk-python-devel vtk-python ufc-devel ffc
BuildPreReq: python-module-viper viper zlib-devel syfi libsyfi-devel
BuildPreReq: python-module-syfi sfc python-module-sfc
BuildPreReq: boost-filesystem-devel boost-program_options-devel
BuildPreReq: boost-math-devel boost-signals-devel
BuildPreReq: libpetsc-%scalar_type-devel gcc-c++ libscotch-devel
BuildPreReq: libsuitesparse-devel libmtl4-devel libamesos-devel
BuildPreReq: libparmetis-devel libslepc-%scalar_type-devel libgmp-devel
BuildPreReq: libgmp_cxx-devel libhdf5-mpi-devel cppunit-devel
BuildPreReq: libpytrilinos-devel libtrilinos-devel python-module-PyTrilinos
BuildPreReq: tetex-latex tetex-dvips /usr/bin/ps2pdf

%description
DOLFIN is the C++/Python interface of FEniCS, providing a consistent PSE
(Problem Solving Environment) for ordinary and partial differential equations.

Features:

  * Simple, consistent and intuitive object-oriented API in C++ or Python
  * Automatic and efficient evaluation of finite element variational forms
  through FFC or SyFi
  * Automatic and efficient assembly of linear systems
  * General families of finite elements, including arbitrary order continuous
  and discontinuous Lagrange finite elements, BDM elements, RT elements, BDFM
  elements, Nedelec elements and Crouzeix-Raviart
  * Arbitrary mixed elements as combination of basic elements, including for
  example Taylor-Hood
  * Discontinuous Galerkin methods including jump terms, averages, and integrals
  over interior mesh facets
  * High-performance linear algebra through uBLAS, PETSc, Trilinos and MTL4
  (experimental) with simple C++ and Python wrappers
  * Experimental support for parallel assembly
  * Simplex meshes in 1D, 2D (triangles), and 3D (tetrahedra), including
  adaptive mesh refinement
  * Multi-adaptive mcG(q)/mdG(q) and mono-adaptive cG(q)/dG(q) ODE solvers
  * Support for a range of input/output formats, including DOLFIN XML, VTK,
  Octave, MATLAB, Diffpack, Exodus II

%package -n lib%name
Summary: Shared library of DOLFIN
Group: System/Libraries
Requires: libslepc-%scalar_type python-module-ufc
Requires: libsuitesparse >= 3.4.0-alt1

%description -n lib%name
DOLFIN is the C++/Python interface of FEniCS, providing a consistent PSE
(Problem Solving Environment) for ordinary and partial differential equations.

This package contains shared library of DOLFIN.

%package -n lib%name-devel
Summary: Development files of DOLFIN
Group: Development/Other
Requires: lib%name = %version-%release
Requires: %oname-devel-common = %version-%release
Requires: libslepc-%scalar_type-devel
Requires: libsyfi-devel ufc-devel zlib-devel
Requires: libparmetis-devel >= 3.1.1-alt5
Requires: libsuitesparse-devel >= 3.4.0-alt1
Conflicts: %oname-devel-common < %version-%release
Obsoletes: %oname-devel-common < %version-%release

%description -n lib%name-devel
DOLFIN is the C++/Python interface of FEniCS, providing a consistent PSE
(Problem Solving Environment) for ordinary and partial differential equations.

This package contains development files of DOLFIN.

%package -n %oname-doc
Summary: Documentation for DOLFIN
Group: Documentation
BuildArch: noarch
Conflicts: %oname-real-doc
Obsoletes: %oname-real-doc

%description -n %oname-doc
DOLFIN is the C++/Python interface of FEniCS, providing a consistent PSE
(Problem Solving Environment) for ordinary and partial differential equations.

This package contains documentation for DOLFIN.

%package -n %oname-examples
Summary: Examples for DOLFIN
Group: Documentation
BuildArch: noarch

%description -n %oname-examples
DOLFIN is the C++/Python interface of FEniCS, providing a consistent PSE
(Problem Solving Environment) for ordinary and partial differential equations.

This package contains examples for DOLFIN.

%package -n %oname-example-data
Summary: Example data for DOLFIN
Group: Sciences/Mathematics
BuildArch: noarch

%description -n %oname-example-data
DOLFIN is the C++/Python interface of FEniCS, providing a consistent PSE
(Problem Solving Environment) for ordinary and partial differential equations.

This package contains example data for DOLFIN.

%package -n python-module-%name
Summary: Python module of DOLFIN
Group: Development/Python
%add_python_lib_path %ldir/python
Requires: libpetsc-%scalar_type >= 3.0.0_p7-alt3
Requires: libslepc-%scalar_type >= 3.0.0_p4-alt2
Requires: lib%name-devel = %version-%release
Requires: viper
Requires: python%__python_version(viper)
%setup_python_module %oname
%py_provides %oname
%if "%__python_version" != "2.5"
%py_requires viper multiprocessing
%endif
Provides: python-module-%oname = %version-%release

%description -n python-module-%name
DOLFIN is the C++/Python interface of FEniCS, providing a consistent PSE
(Problem Solving Environment) for ordinary and partial differential equations.

This package contains Python module of DOLFIN.

%package -n python-module-%{name}_utils
Summary: Utils Python module of DOLFIN
Group: Development/Python
%add_python_lib_path %ldir/python
Requires: libpetsc-%scalar_type >= 3.0.0_p7-alt3
Requires: libslepc-%scalar_type >= 3.0.0_p4-alt2
Requires: lib%name = %version-%release
%setup_python_module %{oname}_utils
%py_provides %{oname}_utils
Provides: python-module-%{oname}_utils = %version-%release

%description -n python-module-%{name}_utils
DOLFIN is the C++/Python interface of FEniCS, providing a consistent PSE
(Problem Solving Environment) for ordinary and partial differential equations.

This package contains utils Python module of DOLFIN.

%package -n %oname-tests
Summary: Tests and benchmarks for DOLFIN
Group: Development/Other
BuildArch: noarch

%description -n %oname-tests
DOLFIN is the C++/Python interface of FEniCS, providing a consistent PSE
(Problem Solving Environment) for ordinary and partial differential equations.

This package contains tests and benchmarks for DOLFIN.

%package -n %oname-devel-common
Summary: Scalar type independent development files of DOLFIN
Group: Development/Other
Conflicts: lib%name-devel < %version-%release
Obsoletes: lib%name-devel < %version-%release
Requires: python-module-numpy >= 1.4.0-alt1.svn20090822

%description -n %oname-devel-common
DOLFIN is the C++/Python interface of FEniCS, providing a consistent PSE
(Problem Solving Environment) for ordinary and partial differential equations.

This package contains Scalar type independent development files of DOLFIN.

%prep
%setup

%install
mpi-selector --set %mpiimpl
source /usr/bin/petsc-%scalar_type.sh

# tune pkgconfigs

sed -i 's|@SCALAR_TYPE@|%scalar_type|g' dolfin/dolfin.pc.in
cp %_pkgconfigdir/petsc-%scalar_type.pc pkgconfig/
cp %_pkgconfigdir/slepc-%scalar_type.pc pkgconfig/
ln -s petsc-%scalar_type.pc pkgconfig/petsc.pc
ln -s slepc-%scalar_type.pc pkgconfig/slepc.pc
export PKG_CONFIG_PATH=$PWD/pkgconfig:$PKG_CONFIG_PATH

for i in zlib boost boost_filesystem boost_program_options
do
	sed -i 's|@MPIDIR@|%mpidir|g' pkgconfig/$i.pc
	sed -i 's|@PETSC_DIR@|%ldir|g' pkgconfig/$i.pc
	sed -i 's|@PYVER@|%__python_version|g' pkgconfig/$i.pc
done

sed -i 's|@PYVER@|%__python_version|g' \
	scons/simula-scons/simula_scons/pkgconfiggenerators/python.py \
	dolfin/scons.cfg \
	dolfin/SConscript

sed -i 's|@PETSC_DIR@|%ldir|g' \
	scons/simula-scons/simula_scons/pkgconfiggenerators/petsc.py \
	scons/simula-scons/simula_scons/pkgconfiggenerators/slepc.py
sed -i 's|@SCALAR_TYPE@|%scalar_type|g' \
	scons/simula-scons/simula_scons/pkgconfiggenerators/petsc.py \
	scons/simula-scons/simula_scons/pkgconfiggenerators/slepc.py \
	site-packages/dolfin/compile_extension_module.py

# pkg versions

BOOSTCONF="$(rpm -q --provides boost-filesystem-devel|grep pkgconfig|wc -l)"
if [ "$BOOSTCONF" = "0" ]
then
	BOOST_VERSION=$(rpm -q --qf "%{VERSION}" boost-devel)
	sed -i "s|^\(Version:\).*|\1 $BOOST_VERSION|" \
		pkgconfig/boost*.pc
else
	cp -f %_pkgconfigdir/boost*.pc pkgconfig/
fi
ZLIB_VERSION=$(rpm -q --qf "%{VERSION}" zlib-devel)
sed -i "s|^\(Version:\).*|\1 $ZLIB_VERSION|" \
	pkgconfig/zlib.pc

INCS="-I$PWD -I%mpidir/include"
LDIRS="-L%mpidir/lib"
RPATHS="-Wl,-R%mpidir/lib:$PETSC_DIR/lib"

function sconsIt() {
	scons $1 \
		withPetscDir=$PETSC_DIR \
		withSlepcDir=$SLEPC_DIR \
		customCxxFlags="$INCS -fno-strict-aliasing" \
		customLinkFlags="$RPATHS $LDIRS"
	cp -f pkgconfig/* scons/pkgconfig/
}

mkdir -p scons/pkgconfig
cp -f pkgconfig/* scons/pkgconfig/
sconsIt configure
sconsIt

scons install DESTDIR=%buildroot
install -d %buildroot%ldir/lib
install -d %buildroot%ldir/include
install -d %buildroot%ldir/python
install -d %buildroot%_datadir/%oname
install -d %buildroot%_pkgconfigdir

rm -f %buildroot%python_sitelibdir/%oname/_cpp.so
for i in $(ls %mpidir/lib/%mpiimpl/*.so); do
	MCAS="$MCAS $i"
done
mpic++ -o %buildroot%python_sitelibdir/%oname/_cpp.so dolfin/swig/cpp_wrap.os \
	-Wl,-R%mpidir/lib:%mpidir/lib/%mpiimpl:$PETSC_DIR/lib -pthread \
	-Wl,--export-dynamic -shared -Ldolfin -L$PETSC_DIR/lib \
	-lxml2 $(pkg-config petsc-%scalar_type --libs) -lpython%__python_version \
	-lscotch -lscotcherr -lcholmod -lcolamd -lumfpack -lamd -llapack -lblas \
	-lgts -lgthread-2.0 -lrt -lgmodule-2.0 -lglib-2.0 -llapack -lblas \
	-lcholmod -lamd -lcolamd -lparmetis -lmetis -lgmpxx -lgmp -lmp -ldolfin \
	-Wl,--no-as-needed $MCAS -L%mpidir/lib -lmpi_cxx -lmpi

mv demo examples
rm -fr bench/fem/backends/run_benchmark.py
rm -f examples/pde/nonlinear-poisson/cpp/plot.py* \
	examples/ode/stiff/cpp/plot.py* \
	$(egrep -R 'solution\ import' examples |awk -F : '{print $1}')
cp -fR examples bench test %buildroot%_datadir/%oname/
for i in $(egrep -R '@PKG_NAME@' %buildroot%_datadir/%oname/|awk -F : '{print $1}')
do
	sed -i 's|@PKG_NAME@|%name|g' $i
done

%if "%scalar_type" == "complex"
pushd %buildroot%_bindir
mv dolfin-convert dolfin-convert-complex
mv dolfin-order dolfin-order-complex
popd
pushd %buildroot%_man1dir
mv dolfin-convert.1.gz dolfin-convert-complex.1.gz
mv dolfin-order.1.gz dolfin-order-complex.1.gz
popd
%endif

mv %buildroot%_libexecdir/*.so* %buildroot%ldir/lib/
mv %buildroot%_includedir/* %buildroot%ldir/include/
cp %_includedir/ufc.h %buildroot%ldir/include/%oname/
cp %_includedir/swig/ufc.i %buildroot%ldir/include/%oname/swig/

rm -f pkgconfig/petsc*.pc pkgconfig/slepc*.pc
if [ "$BOOSTCONF" != "0" ]; then
	rm -f pkgconfig/boost*.pc
fi
cp -f pkgconfig/*.pc %buildroot%_pkgconfigdir
sed -i \
	's|^\(Requires.*slepc\)\ \(.*petsc\)\ \(.*\)|\1-%scalar_type \2-%scalar_type \3|' \
	%oname/%oname.pc
sed -i 's|^\(prefix\).*|\1=%ldir|' %oname/%oname.pc
install -m644 %oname/%oname.pc %buildroot%_pkgconfigdir/%name.pc
install -m644 scons/pkgconfig/gmp.pc \
	%buildroot%_pkgconfigdir

mv %buildroot%python_sitelibdir/* \
	%buildroot%ldir/python/

pushd %buildroot%_pkgconfigdir
rm -f petsc*.pc slepc*.pc %oname.pc
sed -i 's|^\(Requires.*\)\ petsc\ \(.*\)|\1 petsc-%scalar_type \2|' \
	%name.pc
popd

%files
%doc AUTHORS ChangeLog COPYING TODO
%_bindir/*
%_man1dir/*

%files -n lib%name
%ldir/lib/*.so.*

%files -n lib%name-devel
%ldir/lib/*.so
%ldir/include/*
%_pkgconfigdir/%name.pc

%if "%scalar_type" == "real"
%files -n %oname-devel-common
%_pkgconfigdir/*
%exclude %_pkgconfigdir/%name.pc

%files -n %oname-doc
%doc %dir %_docdir/%oname
%doc %_docdir/%oname

%files -n %oname-examples
%dir %_datadir/%oname
%_datadir/%oname/examples

%files -n %oname-tests
%dir %_datadir/%oname
%_datadir/%oname/test
%_datadir/%oname/bench

%files -n %oname-example-data
%dir %_datadir/%oname
%_datadir/%oname/data
%endif

%files -n python-module-%name
%dir %ldir/python
%ldir/python/%oname

%files -n python-module-%{name}_utils
%dir %ldir/python
%ldir/python/%{oname}_utils

%changelog
* Sat Sep 12 2009 Eugeny A. Rostovtsev (REAL) <real at altlinux.org> 0.9.2-alt5.hg20090831
- Snapshot 20090831
- Rebuilt with shared libraries of requirements instead of static

* Sat Aug 29 2009 Eugeny A. Rostovtsev (REAL) <real at altlinux.org> 0.9.2-alt5.hg20090819.1
- Moved pkg-config files for CHOLMOD and UMFPACK into SuiteSparse package

* Tue Aug 18 2009 Eugeny A. Rostovtsev (REAL) <real at altlinux.org> 0.9.2-alt5.hg20090819
- Reformed pkgconfig settings (except boost*, zlib and suitesparse's libs,
  because this packages haven't pkg-config files now)

* Sat Jul 25 2009 Eugeny A. Rostovtsev (REAL) <real at altlinux.org> 0.9.2-alt4
- Renamed scalar type's independent packages

* Sat Jul 25 2009 Eugeny A. Rostovtsev (REAL) <real at altlinux.org> 0.9.2-alt3
- Fixed finding of Scotch and Cholmod for x86_64

* Sat Jul 18 2009 Eugeny A. Rostovtsev (REAL) <real at altlinux.org> 0.9.2-alt2.1
- Added links for UFC headers
- Install python modules into %ldir/python
	instead %python_sitelibdir

* Sat Jul 18 2009 Eugeny A. Rostovtsev (REAL) <real at altlinux.org> 0.9.2-alt2
- Reformed for support real and complex scalars

* Tue Jul 14 2009 Eugeny A. Rostovtsev (REAL) <real at altlinux.org> 0.9.2-alt1
- Initial build for Sisyphus

